local PT = AceLibrary("PeriodicTable-3.0")

SimpleTradeskill = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceHook-2.1")
local core = SimpleTradeskill
local _G = _G

function core:OnEnable()
	self:RegisterEvent("TRADE_SKILL_SHOW")
	self:RegisterEvent("CRAFT_SHOW")
end

function core:TRADE_SKILL_SHOW()
	if not self:IsHooked("TradeSkillFrame_Update") then
		self:SecureHook("TradeSkillFrame_Update", "TRADE_SKILL_UPDATE")
		self:SecureHook("TradeSkillFrame_SetSelection")
	end
	self:TRADE_SKILL_UPDATE()
end

function core:CRAFT_SHOW()
	if not self:IsHooked("CraftFrame_Update") then
		self:SecureHook("CraftFrame_Update", "CRAFT_UPDATE")
		self:SecureHook("CraftFrame_SetSelection")
	end
	self:CRAFT_UPDATE()
end

function core:TradeSkillFrame_SetSelection(id)
	craft = false
	self:UpdateSelection(id, "TradeSkill")
end

function core:CraftFrame_SetSelection(id)
	craft = true
	self:UpdateSelection(id, "Craft")
end

local craft
-- Various function that don't care whether or not we're using a tradeskill or craft
local function GetNumSkills()
	if craft then
		return GetNumCrafts()
	else
		return GetNumTradeSkills()
	end
end

local function GetSkillInfo(index)
	local skillName, skillType, numAvailable, isExpanded, _
	if craft then
		skillName, _, skillType, numAvailable, isExpanded = GetCraftInfo(index)
	else
		skillName, skillType, numAvailable, isExpanded = GetTradeSkillInfo(index)
	end
	return skillName, skillType, numAvailable, isExpanded
end

local function GetSkillRecipeLink(index)
	if craft then
	 	return GetCraftRecipeLink(index)
	else
		return GetTradeSkillRecipeLink(index)
	end
end

local function GetNumReagents(index)
	if craft then
		return GetCraftNumReagents(index)
	else
		return GetTradeSkillNumReagents(index)
	end
end

local function GetReagentInfo(index, id)
	-- returns name, texture, required, have
	if craft then
		return GetCraftReagentInfo(index, id)
	else
		return GetTradeSkillReagentInfo(index, id)
	end
end

local function GetReagentItemLink(index, id)
	if craft then
		return GetCraftReagentItemLink(index, id)
	else
		return GetTradeSkillReagentItemLink(index, id)
	end
end

local function SplitLink(link)
	-- link is "|cffffffff|Hitem:13928:0:0:0:0:0:0:0|h[Grilled Squid]|h|r" or "|cffffffff|Henchant:7421|h[Runed Copper Rod]|h|r"
	return link:match("^.*|H(%l+):(%d+).*|h%[(.*)%]|h|r$")
end

local function GetHowManyCanBeMade(index)
	local buy, bank = 10000, 10000 -- 10000 is just an arbitrarily high number
	local sname,_,base = GetSkillInfo(index)
	local numReagents = GetNumReagents(index)
	for id=1, numReagents, 1 do
		--local itemid = (GetReagentItemLink(index, id)):match("(%d+)")
		local link = GetReagentItemLink(index, id)
		if not link then return base,0,0 end -- item isn't cached
		--Skip vendor-buyable items
		if not PT:ItemInSet(link, "Tradeskill.Mat.BySource.Vendor") then
			local name, texture, required, have = GetReagentInfo(index, id)
			local cap = math.floor(have / required)
			local bankcap = math.floor(GetItemCount(link, true) / required)
			if cap < buy then buy = cap end
			if bankcap < bank then bank = bankcap end
		end
	end
	if buy == 10000 then buy = 0 end
	if bank == 10000 then bank = 0 end
	return base, buy, bank
end
core.GetHowManyCanBeMade = GetHowManyCanBeMade

-- Prettify the tradeskill window
function core:TRADE_SKILL_UPDATE()
	craft = false
	self:UpdateWindow(FauxScrollFrame_GetOffset(TradeSkillListScrollFrame), TRADE_SKILLS_DISPLAYED, "TradeSkillSkill")
end

function core:CRAFT_UPDATE(...)
	if CraftIsPetTraining() then return end
	craft = true
	self:UpdateWindow(FauxScrollFrame_GetOffset(CraftListScrollFrame), CRAFTS_DISPLAYED, "Craft")
end

function core:UpdateWindow(skillOffset, displayed, buttonprefix)
	local numSkills = GetNumSkills()
	--local name, rank, maxRank = GetTradeSkillLine()
	if numSkills > 0 then
		for i=1, displayed, 1 do
			local skillIndex = i + skillOffset
			local skillName, skillType, numAvailable, isExpanded = GetSkillInfo(skillIndex)
			local skillButton = _G[buttonprefix..i]
			if skillIndex <= numSkills then
				if skillType ~= 'header' then
					local base, buy, bank = GetHowManyCanBeMade(skillIndex)
					if (base == 0) and (buy == 0) and (bank == 0) then
						skillButton:SetText(' ' .. skillName)
					else
						skillButton:SetText(' ' .. skillName .. ' [' .. base .. '/V' .. buy .. '/B' .. bank .. ']')
					end
				end
			end
		end
	end
end

function core:UpdateSelection(id, buttonprefix)
	local numReagents = GetNumReagents(id)
	for i=1, numReagents, 1 do
		local link = GetReagentItemLink(id, i)
		if link and PT:ItemInSet(link, "Tradeskill.Mat.BySource.Vendor") then
			local button = _G[buttonprefix..'Reagent'..i..'Name']
			local text = button:GetText()
			if text then
				button:SetText(button:GetText() .. '*')
			end
		end
	end
end
